//+------------------------------------------------------------------+
//|                            Heikenashi_FisherTransform__EA_v3.mq4 |
//|                        Copyright 2013, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Brian M. Lillard"
#include <IBFX - Quick Scripts.mqh>
//----ea is placed on the chart
//----comments display current signal and previous signal based on heiken ashi
//----ea may wait for next signal or place signal immediately
//----ea will be able to hedge if there is a new signal and ea has an order Open
//----ea will trail if there are orders Open
//----each bar ea will read indicator and the numbers and produce a trading operation
//----of course, ea may have visual componants to ease the traders mind of the EA doing its job..
//waht may be needed of the Tick?
//----EA will use up the processor the most by using ticks therefore they may be avoided at all costs.
//waht is needed by the Bar?
//----EA will be making a decision each bar count by the basis that atleast one indicator is used to find a previous signal
//----Orders will be tracked and trailed if need be but otherwise orders will be opened or even closed with hedging option false
//----
//=problems when using strategy tester=
// 1. gets signal one bar late. 
// 2. doesn't open an order.
//    A. doesn't print if it has found an order.
//    B. doesn't print if it has NOT found an order like "No orders are open at this time or EA could not select its order(s)."
// 3. can run slow on strategy tester on speed="31".
//+------------------------------------------------------------------+
//| input parameters                                                 |
//+------------------------------------------------------------------+
extern string Note1="== Heiken Ashi & Fisher Transform =====";
extern   bool   ShowComments?=False;
extern    int   Timeframe=60;
extern    int   Magicnumber=4;
extern string HeikenAshi = "== Heiken Ashi Indicator =====";
extern    int   MaMethod=1, MaPeriod=20, HAbar=1;
extern string Fisher = "== Fisher Transform Indicator =====";
extern    int   Length=6, Price=0, NumBars=0, FTbar=1;
extern string Note2="== User Settings =====";
         bool   HedgeOrders?       = False; //t; never close trade on new signal
extern   bool   RepeatSignals?     = True;  //more orders in signal if one close
extern   bool   NewSignals?        = True;  //just one signal when there new one
extern double   LotSize            = 0.10;
extern double   MaxSpread          = 3.0;
extern    int   PipSlippage        = 3;
extern    int   ServerRetries      = 3;
extern    int   TakeProfit         = 150;
extern    int   StopLoss           = 35;
extern   bool   TrailingProft?     = False; //true
extern    int   TralingStart       = 0;     //35
extern    int   TrailingStop       = 0;     //5
extern string Note3="== Debug Options =====";                        
extern   bool   TickSound          = False;          
extern   bool   OpenSound          = False;     
extern   bool   TrailSound         = False;
//+------------------------------------------------------------------+
//| global variables                                                 |
//+------------------------------------------------------------------+
static datetime Bar1Time;              //bar counter
         double Spreadpips,            /*pips of the spread*/
                stoplossPT,            /*point decimal of the stoploss*/
                takeprofitPT,          /*point decimal of the takeprofit*/
                trailstartPT,          /*point decimal of the trailing start*/
                trailstopPT;           /*point decimal of the trailing stop*/
         string prev_heikenashi="",    /*previous signal of the heiken ashi*/
                heikenashi="",         /*signal of the heiken ashi*/
                fishertransform="",    /*signal of the fisher transform*/
                Signal,                /*signal of the ea*/
                OrdersComment;         //ea order comment
            int SignalCnt;             //count of signals
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----order comment
   OrdersComment="("+Timeframe+")"+Symbol()+"(#"+Magicnumber+")TP("+TakeProfit+")SL("+StopLoss+")"; 
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function(begin)                                     |
//+------------------------------------------------------------------+
int start()
  {
//----
//----BAR--(the first tick will execute BAR tick because it is not equal to Time[1] at first).
//----
   if(Bar1Time!=Time[1])
      { //this is a bars tick which will..
      Bar1Time=Time[1]; //make equal these two variables, so another tick results in returning the start function until a new bar.
      Get_Signal(); //Signal = BUY, SELL or N/A depending on [NewOrders?] user input which evaluates previous bar signal of heiken ashi.
      Get_Digits(); //Based on Digits being 3 or 5; Spreadpips, stoplossPT, takeprofitPT, trailstartPT and trailstopPT are point decimals.
      EA_Comment(); 
      if(TickSound) {PlaySound("tick.wav");} 
      }
   else
      {
      Get_Signal(); //Signal = BUY, SELL or N/A depending on [NewOrders?] user input which evaluates previous bar signal of heiken ashi.
      Get_Digits(); //Based on Digits being 3 or 5; Spreadpips, stoplossPT, takeprofitPT, trailstartPT and trailstopPT are point decimals.
      EA_Comment(); 
      if(TickSound) {PlaySound("tick.wav");} 
      return(0); //exit the start function.
      }
//----
//----proceeding here on is on the BAR tick(every 60 minutes for the H1 chart)----//
//----
//----check if spread is too high to open an order.
   if(Spreadpips>MaxSpread){Print("Spread too high to open an order. "+DoubleToStr(Spreadpips,1)+"pips"); return(0);}
//----
//----check if we have open orders...if we do not then..
//----
   for(int i=(OrdersTotal()-1); i>=0; i--) 
      {//----loop through any open orders
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES)==True) 
         {
         if(OrderType()==OP_BUY && OrderSymbol()==Symbol() && OrderMagicNumber()==Magicnumber && OrderComment()==OrdersComment)
            {//----we found a buy order the ea has created. proceed with a hedge or trail.
            if(Signal=="SELL")
               { //---- ea has found a new signal through RepeatSignals?.
               OpenOrder(false); //open a sell order and possibly hedge.
               }
            else
               { //----no new signal was found so trail BUY if possible.
               //call trail function.
               Print("Nope.");
               }
            }
         else
            {
            if(OrderType()==OP_SELL && OrderSymbol()==Symbol() && OrderMagicNumber()==Magicnumber && OrderComment()==OrdersComment)
               {//---we found a sell order the ea has created. proceed with a hedge or trail.
               if(Signal=="BUY")
                  { //---- ea has found a new signal through RepeatSignals?.
                  OpenOrder(true); //open buy order and possibly hedge.
                  }
               else
                  { //----no new signal was found so trail SELL if possible.
                  //call trail function.
                  Print("Nope.");
                  }
               }
            }
         }
      else
         {//----we have no open orders. open an order using NewSignals? or RepeatSignals?
         Print("No orders are open at this time or EA could not select its order(s).");
         if(Signal=="BUY")
            {
            OpenOrder(true); //open buy order and possibly hedge.
            }
         else
            {
            if(Signal=="SELL")
               {
               OpenOrder(false); //open sell order and possibly hedge.
               }
            }
         }
      }
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function(end)                                       |
//+------------------------------------------------------------------+

//----
void EA_Comment()
   {
   if(ShowComments?) 
      {
      Comment("[Heiken_Ashi_&_Fisher_Transform_E.A._ver.3.0] " + OrdersComment +
       "\n" + "[Heiken Ashi] " + heikenashi + "_(prev=" + prev_heikenashi + ")" +
       "\n" + "[Fisher Transform] " + fishertransform +
       "\n" + "[SPREAD] " + DoubleToStr(Spreadpips, 1) + "//" + DoubleToStr(MaxSpread, 1) +    
       "\n" + "[SIGNAL] " + Signal +
       "\n" + "[SIGNAL COUNT] " + SignalCnt +
       "\n" + "[REPEAT SIGNALS?] " + RepeatSignals? +
       "\n" + "[NEW SIGNALS?] " + NewSignals?
       
       
      );}
   else {Comment("");}
   }
   
//----
void Get_Digits()
   {
   if(Digits==3)
      {
      Spreadpips = (Ask-Bid) * 100;
      stoplossPT = StopLoss * 0.010;
      takeprofitPT = TakeProfit * 0.010;
      trailstartPT = TralingStart * 0.010;
      trailstopPT = TrailingStop * 0.010;
      //double sellpips = (OrderOpenPrice() - Bid) * 100;  
      //double buypips = Ask - OrderOpenPrice() * 100;
      
      }
   if(Digits==5)
      {
      Spreadpips = (Ask-Bid) * 10000;
      stoplossPT = StopLoss * 0.00010;
      takeprofitPT = TakeProfit * 0.00010;
      trailstartPT = TralingStart * 0.00010;
      trailstopPT = TrailingStop * 0.00010;
      //sellpips = (OrderOpenPrice() - Bid) * 10000;
      //buypips = Ask - OrderOpenPrice() * 10000;
      
      }
   }
   
//----
void Get_Signal()
   {
   //Signal=""; //reset
   //"PREVIOUS" HEIKEN ASHI SIGNAL
   if(iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,2,HAbar+1)<iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,3,HAbar+1)){prev_heikenashi="BUY";}
   if(iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,2,HAbar+1)>iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,3,HAbar+1)){prev_heikenashi="SELL";}
   //"CURRENT" HEIKEN ASHI SIGNAL
   if(iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,2,HAbar)<iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,3,HAbar)){heikenashi="BUY";}
   if(iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,2,HAbar)>iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,3,HAbar)){heikenashi="SELL";}
   //"CURRENT" FISHER TRANSFORM SIGNAL
   if(iCustom(NULL,Timeframe,"fisher",Length,Price,NumBars,0,FTbar)<iCustom(NULL,Timeframe,"fisher",Length,Price,NumBars,1,FTbar)){fishertransform="BUY";}
   if(iCustom(NULL,Timeframe,"fisher",Length,Price,NumBars,0,FTbar)>iCustom(NULL,Timeframe,"fisher",Length,Price,NumBars,1,FTbar)){fishertransform="SELL";}
   //----
   if(NewSignals?) // 1st...no orders are open.
      { //---- new signal. the first signal the ea takes upon dropping on a chart.
      if(prev_heikenashi=="SELL" && heikenashi=="BUY" && fishertransform=="BUY" && SignalCnt==0){Signal="BUY"; SignalCnt++; Print("NEW - BUY SIGNAL"); return;}      
      if(prev_heikenashi=="BUY" && heikenashi=="SELL" && fishertransform=="SELL" && SignalCnt==0){Signal="SELL"; SignalCnt++; Print("NEW - SELL SIGNAL"); return;}
      }
   else
      {
      if(heikenashi=="SELL" && fishertransform=="SELL" && Signal!="SELL"){Signal="SELL"; SignalCnt++; Print("NO NEW SIGNAL - SELL SIGNAL");}
      if(heikenashi=="BUY" && fishertransform=="BUY" && Signal!="BUY"){Signal="BUY"; SignalCnt++; Print("NO NEW SIGNAL - BUY SIGNAL");}

      }
   //----
   if(RepeatSignals?) // 2nd..no orders are open
      { //---- repeat signal. there have been atleast one order opened, or signal, and therefore the previous signal must match the current signal.
      if(prev_heikenashi=="SELL" && heikenashi=="SELL" && fishertransform=="SELL" && SignalCnt>=1 && Signal!="SELL"){Signal="SELL"; SignalCnt++; Print("REPEAT - SELL SIGNAL");}
      if(prev_heikenashi=="BUY" && heikenashi=="BUY" && fishertransform=="BUY" && SignalCnt>=1 && Signal!="BUY"){Signal="BUY"; SignalCnt++; Print("REPEAT - BUY SIGNAL");}
      
      }
   else
      { //---- 3rd. ---- ea signal. no previous signal needed.
      if(heikenashi=="SELL" && fishertransform=="SELL" && Signal!="SELL"){Signal="SELL"; SignalCnt++; Print("NO REPEAT - SELL SIGNAL");}
      if(heikenashi=="BUY" && fishertransform=="BUY" && Signal!="BUY"){Signal="BUY"; SignalCnt++; Print("NO REPEAT - BUY SIGNAL");}
      
      }
   //----comment purposes signal
   //if(heikenashi=="SELL" && fishertransform=="SELL" && Signal!="SELL"){Signal="SELL";}
   //if(heikenashi=="BUY" && fishertransform=="BUY" && Signal!="BUY"){Signal="BUY";}
   }
   
//----
void OpenOrder(bool buy)
   {
   if(buy)
      { //---- BUY
      if(HedgeOrders?)
         { //---- hedge(T): leave open BUY order and just open a new BUY order
         EnterLong(Symbol(), LotSize, OrdersComment, StopLoss, TakeProfit, Magicnumber, ServerRetries, PipSlippage); if(OpenSound){PlaySound("ok.wav");}
         Print("Opened a BUY order and ea is hedging...");
         }
      else
         { //---- hedge(F): close open SELL order and then open a new BUY order
         CloseAll(Symbol(), Magicnumber, OP_SELL, ServerRetries, PipSlippage, OrdersComment);
         EnterLong(Symbol(), LotSize, OrdersComment, StopLoss, TakeProfit, Magicnumber, ServerRetries, PipSlippage); 
         if(OpenSound){PlaySound("ok.wav");}
         Print("Closed SELL order and opened a BUY order...");
         }
      //if(Arrows) { SetArrow(Time[0], Ask, 241, Red); }
      //if(VerticalLines) { CreateVLine("VBar"+ObjCnt, true); } 
      }
   else 
      { //---- SELL
      if(HedgeOrders?)
         { //---- hedge(T): leave open orders and just open a new SELL order
         EnterShrt(Symbol(), LotSize, OrdersComment, StopLoss, TakeProfit, Magicnumber, ServerRetries, PipSlippage); if(OpenSound){PlaySound("ok.wav");}
         Print("Opened a SELL order and ea is hedging...");
         }
      else
         { //---- hedge(F): close open BUY order and then open a new SELL order
         CloseAll(Symbol(), Magicnumber, OP_BUY, ServerRetries, PipSlippage, OrdersComment);
         EnterShrt(Symbol(), LotSize, OrdersComment, StopLoss, TakeProfit, Magicnumber, ServerRetries, PipSlippage); 
         if(OpenSound){PlaySound("ok.wav");}
         Print("Closed BUY order and opened a SELL order...");
         }
      //if(Arrows) { SetArrow(Time[0], Bid, 242, Blue); }
      //if(VerticalLines) { CreateVLine("VBar"+ObjCnt, false); }
      }
   }
   
//----


































